local mod = MorgDKP2:NewModule("LDBPlugin", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local C = LibStub("AceConfigDialog-3.0")
local broker = LibStub("LibDataBroker-1.1", true)
local icon = LibStub("LibDBIcon-1.0", true)

local mdkp = MorgDKP2
local db

local defaults = { 
	profile = {
		minimapicon = {
			hide = false
		},
   	},
}

mod.modName = "LDBPlugin"
mod.modref = "LDBPlugin"

function mod:GetOptions()
	return nil
end

function mod:IsDKP()
	return nil
end

function mod:UpdateDB()
	db = mdkp.db.profile
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("LDBPlugin", defaults)
end

function mod:OnEnable()
	db = mdkp.db.profile
	mdkp.db.profile.moduleON["LDBPlugin"] = true
	self:CreateLDBObject()
	if not icon:IsRegistered("MorgDKP2") then icon:Register("MorgDKP2", self.LDBframe.obj, self.db.profile.minimapicon) end
end

function mod:ToggleMinimapIcon(ON)
	if ON then icon:Show("MorgDKP2")
	else icon:Hide("MorgDKP2") end
end

function mod:OnDisable()
	mdkp.db.profile.moduleON["LDBPlugin"] = nil
end

function mod:ActivateCore()
	if not self.LDBframe then return end
	mod.LDBframe.obj.icon = "Interface\\AddOns\\MorgDKP2\\Icons\\AquaON"
	mod.LDBframe.hint:SetText(L["|c000070ddClick:|r MorgBid2 Base query.\n|c000070ddALT-Click:|r Toggle ML/DE\n|c000070ddSHIFT-Click:|r Invite & Waitlist.\n|c000070ddCTRL-Click:|r Raid Tracker.\n|c000070ddCTRL-Click Itemlink:|r Ignore item."])
	self:ToggleFrameUpdate("TooltipLDB", true)
end

function mod:DeActivateCore()
	if not self.LDBframe then return end
	mod.LDBframe.obj.icon = "Interface\\AddOns\\MorgDKP2\\Icons\\AquaOFF"
	mod.LDBframe.hint:SetText(L["On STANDBY.\n|c000070ddClick:|r to enable."])
	self:ToggleFrameUpdate("TooltipLDB", true)
end

function mod:CreateLDBObject()
	if self.LDBframe then return end
	local onenter = function(f) f:Show() end
	local onleave = function(f) f:Hide() end
	self.LDBframe = self:Createframe("Tooltip_BrokerMorgDKP2", onenter, onleave, nil, nil, L["On STANDBY.\n|c000070ddClick:|r to enable."], "TooltipLDB",nil, nil, mdkp.version or L["MorgDKP2"])
	self.LDBframe:SetFrameStrata("FULLSCREEN")
	self.LDBframe:SetClampedToScreen(true)
	self.LDBframe.obj = broker:GetDataObjectByName("MorgDKP2") or broker:NewDataObject("MorgDKP2", {type = "data source", icon = "Interface\\AddOns\\MorgDKP2\\Icons\\AquaOFF"})
	self.LDBframe.obj.type = "data source"
	self.LDBframe.obj.label = "MorgDKP2"
	self.LDBframe.obj.OnClick = function(frame, button)
		if button == "RightButton" then
			mdkp:LoadOptionsSlash()
		else 
			if mod:CoreStatus() then
				if IsAltKeyDown() then
					mdkp.MLmode = not mdkp.MLmode
				elseif IsShiftKeyDown() then
					local core = self:GetModuleRef("CoreModule")
					core:ListDKP("All")
				elseif IsControlKeyDown() then
					local raidtracker = self:GetModuleRef("RaidTracker")
					if raidtracker then raidtracker:OpenFrame() end
				else 
					local core = self:GetModuleRef("CoreModule")
					core:MorgBidQuery()
				end
			else 
				self.LDBframe:Hide()
				mdkp:ActivateCore(true)
			end
			self:Updatetooltipdata(self.LDBframe)
		end
	end
	self.LDBframe.obj.OnEnter = function(frame)
		local point, relPoint
		local dewpoint = "RIGHT"
		local X, Y = self:GetScaledCursorPosition()
		local width = GetScreenWidth() 
		local height = GetScreenHeight() / 2
		if Y < height then point, relPoint = "BOTTOM", "TOP" 
		else point, relPoint = "TOP", "BOTTOM" 
		end
		if X + mdkp.db.profile.media.fontsize * 25 > width then dewpoint = "LEFT" end 
		mdkp.db.profile.dewpoint = dewpoint
		self.LDBframe:ClearAllPoints()
		self.LDBframe:SetPoint(point, frame, relPoint)
		self.LDBframe:Show()
		self:Updatetooltipdata(self.LDBframe)
	end
	self.LDBframe.obj.OnLeave = function()
		self.LDBframe:Hide() 
	end
end

function mod:Updatetooltipdata(f)
	for v = 1, f.textid-1 do
    		f.text[v]:Hide()
    		f.text[v] = nil
	end
	f.textid = 1
	f.textlen = 0
	f.height = 0
	f:Hide()
	local width
	if mdkp.db.profile.moduleON.CoreModule then 
		local db = mdkp.db.profile
		local MLstatustext = L["|c009d9d9dOFF|r"]
		if mdkp.MLmode then MLstatustext = L["|c001eff00ON|r"] end
		local dkpstatus = db.dkpstatustext
		local tracker = mdkp:GetModule("Tracker", true)
		local added
		local onleave = function(f) if not MouseIsOver(f:GetParent()) and (not self.dewframe or (self.dewframe and not MouseIsOver(self.dewframe))) then f:GetParent():Hide() end end
		local onenter = function(f) f:GetParent():Show() end
		
		if db.raidnum ~= 0 and db.raidlog[db.raidnum] and db.raidlog[db.raidnum].loot and #db.raidlog[db.raidnum].loot ~= 0 then
			self:AddTextLine(f)
			for num, loot in pairs(db.raidlog[db.raidnum].loot) do
  				local name = self:GetClasscolors(loot.Player)
  				if loot.ItemLink and loot.Boss ~= "Bank" then
  					local onclick = function() self:TooltipLootClick(loot.ID, loot.ItemLink) end
  					local onenter = function(f) 
								GameTooltip:SetOwner(f, "ANCHOR_LEFT")
								if IsShiftKeyDown() then self:ShowBidSummary(loot.ID, loot.ItemName, loot.Quality, mdkp.db.profile.raidnum)
								else GameTooltip:SetHyperlink(loot.ItemLink) end
								GameTooltip:Show()
								f:GetParent():Show()
							end
					local onleave = function(f) 
								GameTooltip:Hide()
								if not MouseIsOver(f:GetParent()) then f:GetParent():Hide() end
							end
					self:AddTextLine(f, onclick, loot.ItemLink, name, nil, nil, loot.Icon, nil, nil, onenter, onleave)
					added = true
				end
			end
			if added then self:AddTextLine(f) end
		end
		
		local onclick = function(f) self:Opendewdrop(f, {"master", "group", "freeforall"}, f.type, "lootmethod") end
		self:AddTextLine(f, onclick, L["Master Looter"], self:GetClasscolors(db.mlooter), nil, nil, nil, nil, disabled, onenter, onleave)
		
		local onclick = function(f) self:Opendewdrop(f, db.raidmembers, f.type, "disenchanter", true) end
		self:AddTextLine(f, onclick, L["Disenchanter"], self:GetClasscolors(db.disenchanter), nil, nil, nil, nil, disabled, onenter, onleave)
		
		local onclick = function(f) self:ToggleML(f:GetParent()) end
		self:AddTextLine(f, onclick, L["ML/DE mode"], MLstatustext, nil, nil, nil, nil, disabled, onenter, onleave)
	
		local onclick = function(f) self:Opendewdrop(f, db.raidlist, f.type, "raid") end
		local r,g,b = 0, 1, 0
		if db.raid == L["NONE"] then r,g,b = 1,0,0 end
		local text = {n = db.raid, r = r, g = g, b = b} 
		self:AddTextLine(f, onclick, L["Main DKP Pool"], text, nil, nil, nil, nil, disabled, onenter, onleave)
	
		local onclick = function(f) self:Opendewdrop(f, mdkp.dkpmenu, f.type, "dkpstatustext") end
		self:AddTextLine(f, onclick, L["DKP System"], dkpstatus, nil, true, nil, nil, disabled, onenter, onleave)
	
		local onclick = function(f) self:Opendewdrop(f, mdkp.Quality, f.type, "quality") end
		self:AddTextLine(f, onclick, L["Quality Threshold"], mdkp.Quality[db.quality + 1], nil, nil, nil, nil, disabled, onenter, onleave)
	
		self:AddTextLine(f)
	
		local r,g,b = 0,1,0
		if db.raidstart == L["PENDING"] then r,g,b = 1,1,0 end
		local text = {n = db.raidstart, r = r, g = g, b = b}
		local onclick = function(f) tracker:BeginRaid(f:GetParent()) mod:Updatetooltipdata(mod.LDBframe) end
		local disabled = nil
		if db.InRaid then disabled = true end
		self:AddTextLine(f, onclick, L["Start Raid"], text, nil, nil, nil, nil, disabled, onenter, onleave)
	
		local r,g,b = 0,1,0
		if db.raidend == L["PENDING"] then r,g,b = 1,1,0 end
		local text = {n = db.raidend, r = r, g = g, b = b}
		local onclick = function(f) 
					local direct
					if IsShiftKeyDown() then direct = true end 
					tracker:ShowEndRaidFrame(f:GetParent(), direct) 
					mod:Updatetooltipdata(mod.LDBframe)
				end
		local disabled = nil
		if not db.InRaid then disabled = true end
		self:AddTextLine(f, onclick, L["End Raid"], text, nil, nil, nil, nil, disabled, onenter, onleave)
	
		local r,g,b = 0,1,0
		if db.raidexport == L["PENDING"] then r,g,b = 1,1,0 end
		local text = {n = db.raidexport, r = r, g = g, b = b}
		local onclick = function(f) tracker:ExportRaids(f:GetParent()) mod:Updatetooltipdata(mod.LDBframe) end
		self:AddTextLine(f, onclick, L["Export Raid"], text, nil, nil, nil, nil, nil, onenter, onleave)
		
		local onclick = function(f) 
					if IsShiftKeyDown() then tracker:CustomEvent()
					else self:Opendewdrop(f, mdkp.ZoneBosses, f.type, "custom") 
					end
					mod:Updatetooltipdata(mod.LDBframe)
				end
		self:AddTextLine(f, onclick, L["Custom Event"], db.custom, nil, nil, nil, nil, disabled, onenter, onleave)
	
		if mdkp.db.profile.modules.DKPovertime then
			local DKPovertime = self:GetModuleRef("DKPovertime")
			if DKPovertime.db.profile.dkpimmed then
				local onclick = function(f) DKPovertime:AwardNow() end
				self:AddTextLine(f, onclick, nil, nil, L["Award DKP Now"], nil, nil, nil, disabled, onenter, onleave)
			end
		end	
		
		width = f.hint:GetStringWidth()/4 + 10
		if width < f.textlen then width = f.textlen end
		f.hint:SetWidth(width)
    		f:SetHeight(f.height + f.hint:GetStringHeight() + 25)
    		for v = 1, f.textid-1 do
    			f.text[v]:SetWidth(width)
    			f.text[v]:Show()
		end
	else 
    		width = f.hint:GetStringWidth() + 15
    		f.hint:SetWidth(width)
    		f.height = f.height + f.hint:GetStringHeight()
    		if f.title then f.height = f.height + f.title:GetStringHeight() + 5 end
    		f:SetHeight(f.height + 10)
    	end
    	f:SetWidth(width+5)
    	f:Show()
end

function mod:TooltipLootClick(itemID, itemlink)
	if IsShiftKeyDown() then
		local ChatFrameEditBox = ChatEdit_GetActiveWindow()
		if ChatFrameEditBox then ChatFrameEditBox:Insert(itemlink) end
	elseif IsControlKeyDown() then
		mdkp.db.profile.ignore[itemID] = true
		self:out(string.format(L["Added %s to ignore list."], itemlink))
	else 
		local loot = self:GetModuleRef("Lootframe")
		if loot then loot:AddManualItem(itemlink) end
	end
end

function mod:ToggleML(f)
	mdkp.MLmode = not mdkp.MLmode
	self:Updatetooltipdata(f)
end
