local mod = MorgDKP2:NewModule("Relational")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")

local mdkp = MorgDKP2 

mod.modName = L["Relational DKP"]
mod.modref = "Relational"

local Player = UnitName("player")
local db
local fmt = string.format
local floor = math.floor

local defaults = { 
	profile = {
		minDKP = 1,
		baseDKP = 0,
	},
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return true
end

function mod:GetName()
	return mod.modName, mod.modref
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("Relational", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile	
	db.moduleON["Relational"] = true
	
end

function mod:OnDisable()
	db.moduleON["Relational"] = nil
end

function mod:UpdateDB()
	db = mdkp.db.profile
end

function mod:Toggle(info, v)
	mdkp.db.profile.modules.Relational = v
	if v then mod:Enable()
	else mod:Disable() end
	self:DKPChange(mod.modName)
end

function mod:SetOpt(info, v)
	local db = mod.db.profile
	local opt = info[#info]
	db[opt] = v
end

function mod:GetOpt(info)
	local db = mod.db.profile
	local opt = info[#info]
	return db[opt]
end

function mod:GetPlayerPoints(name, POOL, showearned)
	if not showearned or showearned == 1 then
		local minDKP = mod.db.profile.minDKP
		local baseDKP = mod.db.profile.baseDKP
		local earned, spent = db.info[name][POOL].earned, db.info[name][POOL].spent
		if (spent + baseDKP) < minDKP then spent = minDKP end
		local points = fmt("%.2f", (earned / (spent + baseDKP)))
		return points 
	end
	if showearned == 2 then return db.info[name][POOL].earned
	elseif showearned == 3 then return db.info[name][POOL].attend
	end
end

function mod:GetItemValue(itemid, player, POOL)
	local itempoints = db.items[itemid].points
	itempoints = floor(self:GetTAKEModeValue(itemid, itempoints, player) + 0.5)
	return itempoints
end

function mod:ItemRecorded(id, looter, POOL, points, itempoints, link)
	if looter == L["Bank"] then return end
	points = db.info[looter][POOL].points - itempoints
	self:AwardItem(id, looter, POOL, points, itempoints, link)
end

function mod:ItemRebate(id, looter, POOL, itempoints, link)
	if not id or looter == L["Bank"] then return end
	self:AdjustDKPPoints(looter, looter, POOL, nil, itempoints)
end
